/*
    NullBot Robot Platform - Server - 2005-2011 Jason Hunt
    
    GNU GENERAL PUBLIC LICENSE Version 2, June 1991
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    Contact: nulluser@gmail.com
    
    File: utility.c
*/


#include <string.h>
#include <errno.h>
#include <stdio.h>
#include <stdarg.h>

#include "utility.h"

#define MODULE_NAME "[Utility] "

unsigned char logging = 0;

/* Write formatted data to the log file */
void log_write( char *fmt, ... )
{
    if (!logging) return;

    static int log_init = 0;
    static FILE * log_file = NULL;
       
    if (!log_init)
    {
        log_file = fopen("log.txt", "wt");
        
        log_init = 1;
    }

    const unsigned int max = 255;

    va_list args;
    char text[max];

    va_start(args, fmt);
    vsnprintf(text, max, fmt, args);
    fprintf(log_file, "%s", text);

    va_end(args);    
    
    fflush(log_file); 
}
/* End of log_write */


/* Write formatted data to the log file */
void alert( char *fmt, ... )
{
    const unsigned int max = 255;

    va_list args;
    char text[max];

    va_start(args, fmt);
    vsnprintf(text, max, fmt, args);
        
    printf("%s", text);
    log_write("%s", text);
    
    va_end(args);    
}
/* End of log_write */



/* Write formatted data to the log file */
void error( char *fmt, ... )
{
    const unsigned int max = 255;

    va_list args;
    char text[max];

    va_start(args, fmt);
    vsnprintf(text, max, fmt, args);

    printf("ERROR ");      
        
    printf("%s", text);

    printf(" %s\n", strerror (errno));        

    log_write("%s", text);
    
    va_end(args);    
}
/* End of log_write */





/* Log a packet */
void log_packet( char *m, unsigned char *p, unsigned int l )
{
    unsigned int i;

    log_write("%s ", m);

    for (i = 0; i < l; i++)
        log_write("%02x ", p[i]);

    log_write("\n");
}
/* End of log_packet */


/* Log a packet */
void show_packet( char *m, unsigned char *p, unsigned int l )
{
    unsigned int i;

    alert("%s ", m);

    for (i = 0; i < l; i++)
        alert("%02x ", p[i]);

    alert("\n");
}
/* End of log_packet */


/* Compute a checksum */
unsigned int get_checksum( unsigned char *bf, unsigned int len )
{
	unsigned char ck = 0;
	
	while(len)
	{
		ck += bf[--len];
	}

	return(ck);
}
/* End of compute checksum */


/* Get integer from buffer */
int get_int( unsigned char lv, unsigned char hv )
{
    short v = lv + hv * 256;

    return(v);
}
/* End of get int */




